@@ -1,4 +1,5 @@
 from ..nba.constants import *
+import types
 from enum import Enum
 from inspect import isclass
 
@@ -61,10 +62,18 @@ class StatsNbaApi:
         self.specs = json.loads(pkg_resources.read_text('sportsdata.data', 'stats.nba.com.json'))
         self.logger.info(self.specs.keys())
 
+        self.parameters = {}
+        for param in self.specs['parameters']:
+            self.parameters[param['name']] = {
+                'default': param['default'],
+                'values': param['values']
+            }
+
+        # Create methods for each api endpoint
         for endpoint in self.specs['stats_endpoints']:
             self.add_api_method(endpoint)
 
-    def add_api_method(cls, endpoint):
+    def add_api_method(self, endpoint):
         """
         Dynamically builds a method for each endpoint in the specification file
         :param endpoint:
@@ -73,26 +82,61 @@ class StatsNbaApi:
         :return:
         """
 
-        def dynamic_method(self, **kwargs):
+        def dynamic_method(self2, **kwargs):
             url = endpoint['url']
             parameters = endpoint['parameters']
-            self.logger.info(url)
-            # self.logger.info(kwargs)
-            return url, parameters, kwargs
-            #return requests.get(url, params=parameters)
+            self2.logger.info(url)
+            self.logger.info(kwargs)
+
+            # Determine the ResponseType
+            return_type = ReturnType.DICTIONARY.value
+            if 'ReturnType' in kwargs:
+                return_type = kwargs['ReturnType']
+
+            url_parameters = {}
+            # Add each parameter to the url
+            for param in parameters:
+                value = ''
+
+                # Was this parameter passed into to the method?
+                if param in kwargs:
+                    value = kwargs[param]
+
+                # Does this parameter have a default value
+                elif param in self.parameters:
+                    value = self.parameters[param]['default']
+
+                # Is the value for the parameter a legal value for this parameter
+                if param in self.parameters and value not in self.parameters[param]['values']:
+                    self.logger.warning(f"The value '{value}' is not a legal value for '{param}'")
+                    value = ''
+
+                # Add the parameter and its value to the dictionary of url parameters
+                url_parameters[param] = value
+
+            self.logger.info(url_parameters)
+            response = requests.get(url, params=url_parameters, timeout=10)
+
+            if return_type == ReturnType.DICTIONARY or return_type == ReturnType.DICTIONARY.value:
+                return_value = ResponseParser.get_dictionary(response)
+            elif return_type == ReturnType.RESPONSE or return_type == ReturnType.RESPONSE.value:
+                return_value = response
+            elif return_type == ReturnType.DATA_FRAMES or return_type == ReturnType.DATA_FRAMES.value:
+                return_value = ResponseParser.get_data_frames(response)
+
+            return return_value
 
         dynamic_method.__name__ = endpoint['name']
-        setattr(cls, dynamic_method.__name__, classmethod(dynamic_method))
+        setattr(self, dynamic_method.__name__, types.MethodType(dynamic_method, self))
 
     base_url = "https://stats.nba.com/stats/{0}"
     headers = {
         'user-agent': (
             'Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/57.0.2987.133 Safari/537.36'),
-        # noqa: E501
-        'Dnt': ('1'),
-        'Accept-Encoding': ('gzip, deflate, sdch'),
-        'Accept-Language': ('en'),
-        'origin': ('https://stats.nba.com')
+        'Dnt': '1',
+        'Accept-Encoding': 'gzip, deflate, sdch',
+        'Accept-Language': 'en',
+        'origin': 'https://stats.nba.com'
     }
 
     # @clean_inputs
